#include "vymview.h"

#include <iostream>
using namespace std;

#include "linkablemapobj.h"
#include "mainwindow.h"
#include "mapeditor.h"
#include "treeeditor.h"

extern Main *mainWindow;


VymView::VymView(VymModel *m)
{
	model=m;

	// Create TreeView
	treeEditor=new TreeEditor (model);
	treeEditor->setModel ((QAbstractItemModel*)model);
	//treeEditor->setMinimumWidth (50);

	treeEditor->setColumnWidth (0,350);

	selModel=treeEditor->selectionModel();
	model->setSelectionModel (selModel);
	connect (
		selModel, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
		this,SLOT (changeSelection(const QItemSelection &,const QItemSelection &)));

	// Create good old MapEditor
	mapEditor=model->getMapEditor();
	if (!mapEditor) mapEditor=new MapEditor (model);
	connect (
		selModel, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
		mapEditor,SLOT (updateSelection(const QItemSelection &,const QItemSelection &)));
		/*
	connect (
		selModel, SIGNAL (currentChanged(const QModelIndex &, const QModelIndex &)), 
		me,SLOT (updateCurrent(const QModelIndex &,const QModelIndex &)));
		*/
		/*
*/
	connect (
		model, SIGNAL (dataChanged(const QModelIndex &, const QModelIndex &)), 
		mapEditor,SLOT (updateData(const QModelIndex &) ) );

	// VymModel may want to update selection or other data, e.g. during animation
	connect (
		model, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
		mapEditor,SLOT (updateSelection(const QItemSelection &,const QItemSelection &)));

	connect (
		model, SIGNAL (noteHasChanged(QModelIndex) ),
		mainWindow, SLOT (updateNoteEditor (QModelIndex) ) );
		
	connect (
		model, SIGNAL (expandAll() ),
		this, SLOT (expandAll () ) );
		
	connect (
		model, SIGNAL (showSelection() ),
		this, SLOT (showSelection() ) );
		

	mapEditor->setAntiAlias (mainWindow->isAliased());
	mapEditor->setSmoothPixmap(mainWindow->hasSmoothPixmapTransform());

	addWidget (treeEditor);
	addWidget (mapEditor);

	// Set geometry
	QList <int> widths;
	widths<<120;
	widths<<600;
	setSizes(widths);
}

void VymView::initFocus()
{
	mapEditor->setFocus();
}

QItemSelectionModel* VymView::selectionModel() 
{
	if (treeEditor) 
		return selModel;
	else 
		std::cout <<"VymView::selectionModel: hey, no treeEditor so far???\n";
	return NULL;
}

void VymView::changeSelection (const QItemSelection &newsel, const QItemSelection &oldsel)
{
	/*
	cout <<"VymView::changeSelection (";
	if (!newsel.indexes().isEmpty() )
		cout << model->getItem(newsel.indexes().first() )->getHeading().toStdString();
	cout << " <- ";
	if (!oldsel.indexes().isEmpty() )
		cout << model->getItem(oldsel.indexes().first() )->getHeading().toStdString();
	cout << ")\n";
	*/

	// Notify mainwindow to update satellites like NoteEditor, if needed (model==currenModel...)
	mainWindow->changeSelection (model,newsel,oldsel);	// FIXME-3 maybe connect VymModel <-> MainWindow directly?
	showSelection();
}

void VymView::expandAll()
{
	treeEditor->expandAll();
}

void VymView::showSelection()
{
	treeEditor->scrollTo(
		model->getSelectedIndex(), 
		//QAbstractItemView::PositionAtCenter   
		QAbstractItemView::EnsureVisible
	);

	LinkableMapObj* lmo=model->getSelectedLMO();
	if (lmo) 
		mapEditor->setScrollBarPosTarget (lmo->getBBox() );
}

